// Copyright (C) 1999 Jean-Marc Valin
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this file.  If not, write to the Free Software Foundation,
// 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef FRAME_UNARY_OPERATION_H
#define FRAME_UNARY_OPERATION_H

#include "Node.h"
#include "FrameOperation.h"

/**Class for nodes that operate on frames (typically audio) using a single input and a single output*/
class FrameUnaryOperation : public FrameOperation {
protected:
   /**Length of input frames*/
   int inputLength;

   /** inputID*/
   int inputID;

   /**inputBuffer*/
   ObjectRef inputBuffer;
public:
   /**Constructor, takes the name of the node and a set of parameters*/
   FrameUnaryOperation(string nodeName, const ParameterSet &params);

   /**Class specific initialization routine.
      Each class will call its subclass specificInitialize() method*/
   virtual void specificInitialize();

   /**Class reset routine.
      Each class will call its superclass reset() method*/
   virtual void reset();

   /**Ask for the node's output which ID (number) is output_id 
      and for the 'count' iteration */
   virtual void calculate(int output_id, int count, Buffer &out); 

protected:
   /**Default constructor, should not be used*/
   FrameUnaryOperation() {throw new GeneralException("FrameUnaryOperation copy constructor should not be called",__FILE__,__LINE__);}

   /**Computes the output for one frame*/
   virtual void computeFrame(ObjectRef input, int count) = 0;

};

#endif
