// Copyright (C) 1999 Dominic Letourneau and Andre Charbonneau
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this file.  If not, write to the Free Software Foundation,
// 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef FileParserException_h
#define FileParserException_h

/*
** Includes
*/
#include "Exception.h"

/*
** Defines
*/
#define NO_TYPE -1
#define FATAL_ERROR 0
#define IO_EXCEPTION 1
#define SYNTAX_EXCEPTION 2
#define NOT_FOUND_EXCEPTION 3
#define DUPLICATE_EXCEPTION 4

/**
 ** The FileParserException class implements exception handling for everything that
 ** is relating to data-flow network file parsing.
 */
class FileParserException : public BaseException
{
 private:
  /** The type of the exception. Valid types are:
   ** NO_TYPE,FATAL_ERROR,IO_EXCEPTION,SYNTAX_EXCEPTION,NOT_FOUND_EXCEPTION,DUPLICATE_EXCEPTION
   */
  int m_type;

  /// The description of the exception.
  string m_description;


 public:
  /** Constructor
   ** @param type The type of the exception. Must be in {NO_TYPE,FATAL_ERROR,IO_EXCEPTION,SYNTAX_EXCEPTION,NOT_FOUND_EXCEPTION,DUPLICATE_EXCEPTION}
   ** @param description The description of the exception. (optional)
   */
  FileParserException(int type, const string& description = "");



  /** This method is used to print the exception to a stream, cerr by default.
   ** @param out The stream to print to. (default to cerr)
   ** @return void
   */
  virtual void print(ostream &out = cerr);


  /** Gets the description of the exception.
   ** @return The description as a string.
   */
  string get_description() const;


  /** Gets the type of the exception.
   ** @return The description's type as an integer.
   */
  int get_type() const;

};

#endif
